@@ -1,10 +1,9 @@
-from _patch import zmq, USING_GEVENT  # NOQA
+from _patch import zmq  # NOQA
 import logging
 import os
-import sys
 
 
-version_info = (0, 6, 0)
+version_info = (0, 7, 0)
 __version__ = ".".join(map(str, version_info))
 
 
@@ -21,9 +20,8 @@ def get_arbiter(watchers, controller=None,
 
     Options:
 
-    - **watchers** -- a list of watchers. A watcher in that case is a
-      dict containing:
-
+      - **watchers** -- a list of watchers. A watcher in that case is a
+                        dict containing:
         - **name** -- the name of the watcher (default: None)
         - **cmd** -- the command line used to run the Watcher.
         - **args** -- the args for the command (list or string).
@@ -48,9 +46,9 @@ def get_arbiter(watchers, controller=None,
           the stdout stream. Default to None. When provided, may contain:
 
             - **class**: the fully qualified name of the class to use for
-              streaming. Defaults to circus.stream.FileStream
+                         streaming. Defaults to circus.stream.FileStream
             - **refresh_time**: the delay between two stream checks. Defaults
-              to 0.3 seconds.
+                                to 0.3 seconds.
             - any other key will be passed the class constructor.
         - **stderr_stream**: a mapping containing the options for configuring
           the stderr stream. Default to None. When provided, may contain:
@@ -58,7 +56,7 @@ def get_arbiter(watchers, controller=None,
             - **class**: the fully qualified name of the class to use for
               streaming. Defaults to circus.stream.FileStream
             - **refresh_time**: the delay between two stream checks. Defaults
-              to 0.3 seconds.
+                                to 0.3 seconds.
             - any other key will be passed the class constructor.
         - **max_retry**: the number of times we attempt to start a process,
           before we abandon and stop the whole watcher. (default: 5)
@@ -96,15 +94,6 @@ def get_arbiter(watchers, controller=None,
     if pubsub_endpoint is None:
         pubsub_endpoint = DEFAULT_ENDPOINT_SUB
 
-    if stream_backend == 'gevent':
-        try:
-            import gevent                   # NOQA
-        except ImportError:
-            sys.stderr.write("stream_backend set to gevent, " +
-                             "but gevent isn't installed\n")
-            sys.stderr.write("Exiting...")
-            sys.exit(1)
-
     from circus.watcher import Watcher
     if background:
         from circus.arbiter import ThreadedArbiter as Arbiter   # NOQA
